@@ -5,9 +5,9 @@
 Various utilities for imageio
 """
 
-from __future__ import absolute_import, print_function, division
-
 
+from collections import OrderedDict
+import numpy as np
 import os
 import re
 import struct
@@ -15,38 +15,22 @@ import sys
 import time
 import logging
 
+
 logger = logging.getLogger("imageio")
 
-
-import numpy as np
-
 IS_PYPY = "__pypy__" in sys.builtin_module_names
 THIS_DIR = os.path.abspath(os.path.dirname(__file__))
 
-# Taken from six.py
-PY3 = sys.version_info[0] == 3
-if PY3:
-    string_types = (str,)
-    text_type = str
-    binary_type = bytes
-else:  # pragma: no cover
-    string_types = (basestring,)  # noqa
-    text_type = unicode  # noqa
-    binary_type = str
-
 
 def urlopen(*args, **kwargs):
-    """ Compatibility function for the urlopen function. Raises an
+    """Compatibility function for the urlopen function. Raises an
     RuntimeError if urlopen could not be imported (which can occur in
     frozen applications.
     """
     try:
-        from urllib2 import urlopen
+        from urllib.request import urlopen
     except ImportError:
-        try:
-            from urllib.request import urlopen  # Py3k
-        except ImportError:
-            raise RuntimeError("Could not import urlopen.")
+        raise RuntimeError("Could not import urlopen.")
     return urlopen(*args, **kwargs)
 
 
@@ -59,7 +43,7 @@ def _precision_warn(p1, p2, extra=""):
 
 
 def image_as_uint(im, bitdepth=None):
-    """ Convert the given image to uint (default: uint8)
+    """Convert the given image to uint (default: uint8)
 
     If the dtype already matches the desired format, it is returned
     as-is. If the image is float, and all values are between 0 and 1,
@@ -125,7 +109,7 @@ def image_as_uint(im, bitdepth=None):
 
 
 class Array(np.ndarray):
-    """ Array(array, meta=None)
+    """Array(array, meta=None)
 
     A subclass of np.ndarray that has a meta attribute. Get the dictionary
     that contains the meta data using ``im.meta``. Convert to a plain numpy
@@ -140,7 +124,7 @@ class Array(np.ndarray):
         if not (meta is None or isinstance(meta, dict)):
             raise ValueError("Array expects meta data to be a dict.")
         # Convert and return
-        meta = meta if meta is not None else {}
+        meta = meta if meta is not None else getattr(array, "meta", {})
         try:
             ob = array.view(cls)
         except AttributeError:  # pragma: no cover
@@ -150,8 +134,7 @@ class Array(np.ndarray):
         return ob
 
     def _copy_meta(self, meta):
-        """ Make a 2-level deep copy of the meta dictionary.
-        """
+        """Make a 2-level deep copy of the meta dictionary."""
         self._meta = Dict()
         for key, val in meta.items():
             if isinstance(val, dict):
@@ -160,12 +143,11 @@ class Array(np.ndarray):
 
     @property
     def meta(self):
-        """ The dict with the meta data of this image.
-        """
+        """The dict with the meta data of this image."""
         return self._meta
 
     def __array_finalize__(self, ob):
-        """ So the meta info is maintained when doing calculations with
+        """So the meta info is maintained when doing calculations with
         the array.
         """
         if isinstance(ob, Array):
@@ -174,13 +156,15 @@ class Array(np.ndarray):
             self._copy_meta({})
 
     def __array_wrap__(self, out, context=None):
-        """ So that we return a native numpy array (or scalar) when a
+        """So that we return a native numpy array (or scalar) when a
         reducting ufunc is applied (such as sum(), std(), etc.)
         """
         if not out.shape:
             return out.dtype.type(out)  # Scalar
         elif out.shape != self.shape:
             return out.view(type=np.ndarray)
+        elif not isinstance(out, Array):
+            return Array(out, self.meta)
         else:
             return out  # Type Array
 
@@ -189,7 +173,7 @@ Image = Array  # Alias for backwards compatibility
 
 
 def asarray(a):
-    """ Pypy-safe version of np.asarray. Pypy's np.asarray consumes a
+    """Pypy-safe version of np.asarray. Pypy's np.asarray consumes a
     *lot* of memory if the given array is an ndarray subclass. This
     function does not.
     """
@@ -201,11 +185,8 @@ def asarray(a):
     return np.asarray(a)
 
 
-from collections import OrderedDict
-
-
 class Dict(OrderedDict):
-    """ A dict in which the keys can be get and set as if they were
+    """A dict in which the keys can be get and set as if they were
     attributes. Very convenient in combination with autocompletion.
 
     This Dict still behaves as much as possible as a normal dict, and
@@ -241,17 +222,17 @@ class Dict(OrderedDict):
             self[key] = val
 
     def __dir__(self):
-        isidentifier = lambda x: bool(re.match(r"[a-z_]\w*$", x, re.I))
-        names = [
-            k for k in self.keys() if (isinstance(k, string_types) and isidentifier(k))
-        ]
+        def isidentifier(x):
+            return bool(re.match(r"[a-z_]\w*$", x, re.I))
+
+        names = [k for k in self.keys() if (isinstance(k, str) and isidentifier(k))]
         return Dict.__reserved_names__ + names
 
 
 class BaseProgressIndicator(object):
-    """ BaseProgressIndicator(name)
+    """BaseProgressIndicator(name)
 
-    A progress indicator helps display the progres of a task to the
+    A progress indicator helps display the progress of a task to the
     user. Progress can be pending, running, finished or failed.
 
     Each task has:
@@ -275,10 +256,10 @@ class BaseProgressIndicator(object):
         self._last_progress_update = 0
 
     def start(self, action="", unit="", max=0):
-        """ start(action='', unit='', max=0)
+        """start(action='', unit='', max=0)
 
         Start the progress. Optionally specify an action, a unit,
-        and a maxium progress value.
+        and a maximum progress value.
         """
         if self._status == 1:
             self.finish()
@@ -291,7 +272,7 @@ class BaseProgressIndicator(object):
         self._start()
 
     def status(self):
-        """ status()
+        """status()
 
         Get the status of the progress - 0: pending, 1: in progress,
         2: finished, 3: failed
@@ -299,7 +280,7 @@ class BaseProgressIndicator(object):
         return self._status
 
     def set_progress(self, progress=0, force=False):
-        """ set_progress(progress=0, force=False)
+        """set_progress(progress=0, force=False)
 
         Set the current progress. To avoid unnecessary progress updates
         this will only have a visual effect if the time since the last
@@ -327,14 +308,14 @@ class BaseProgressIndicator(object):
         self._update_progress(progressText)
 
     def increase_progress(self, extra_progress):
-        """ increase_progress(extra_progress)
+        """increase_progress(extra_progress)
 
         Increase the progress by a certain amount.
         """
         self.set_progress(self._progress + extra_progress)
 
     def finish(self, message=None):
-        """ finish(message=None)
+        """finish(message=None)
 
         Finish the progress, optionally specifying a message. This will
         not set the progress to the maximum.
@@ -346,7 +327,7 @@ class BaseProgressIndicator(object):
             self._write(message)
 
     def fail(self, message=None):
-        """ fail(message=None)
+        """fail(message=None)
 
         Stop the progress with a failure, optionally specifying a message.
         """
@@ -357,7 +338,7 @@ class BaseProgressIndicator(object):
         self._write(message)
 
     def write(self, message):
-        """ write(message)
+        """write(message)
 
         Write a message during progress (such as a warning).
         """
@@ -383,7 +364,7 @@ class BaseProgressIndicator(object):
 
 
 class StdoutProgressIndicator(BaseProgressIndicator):
-    """ StdoutProgressIndicator(name)
+    """StdoutProgressIndicator(name)
 
     A progress indicator that shows the progress in stdout. It
     assumes that the tty can appropriately deal with backspace
@@ -428,7 +409,7 @@ class StdoutProgressIndicator(BaseProgressIndicator):
 
 # From pyzolib/paths.py (https://bitbucket.org/pyzo/pyzolib/src/tip/paths.py)
 def appdata_dir(appname=None, roaming=False):
-    """ appdata_dir(appname=None, roaming=False)
+    """appdata_dir(appname=None, roaming=False)
 
     Get the path to the application directory, where applications are allowed
     to write user specific files (e.g. configurations). For non-user specific
@@ -478,17 +459,14 @@ def appdata_dir(appname=None, roaming=False):
             appname = "." + appname.lstrip(".")  # Make it a hidden directory
         path = os.path.join(path, appname)
         if not os.path.isdir(path):  # pragma: no cover
-            try:
-                os.makedirs(path)
-            except Exception:
-                pass
+            os.makedirs(path, exist_ok=True)
 
     # Done
     return path
 
 
 def resource_dirs():
-    """ resource_dirs()
+    """resource_dirs()
 
     Get a list of directories where imageio resources may be located.
     The first directory in this list is the "resources" directory in
@@ -513,7 +491,7 @@ def resource_dirs():
 
 
 def resource_package_dir():
-    """ package_dir
+    """package_dir
 
     Get the resources directory in the imageio package installation
     directory.
@@ -542,7 +520,7 @@ def resource_package_dir():
 
 
 def get_platform():
-    """ get_platform()
+    """get_platform()
 
     Get a string that specifies the platform more specific than
     sys.platform does. The result can be: linux32, linux64, win32,
@@ -555,6 +533,8 @@ def get_platform():
         plat = "win%i"
     elif sys.platform.startswith("darwin"):
         plat = "osx%i"
+    elif sys.platform.startswith("freebsd"):
+        plat = "freebsd%i"
     else:  # pragma: no cover
         return None
 
@@ -562,8 +542,7 @@ def get_platform():
 
 
 def has_module(module_name):
-    """Check to see if a python module is available.
-    """
+    """Check to see if a python module is available."""
     if sys.version_info > (3, 4):
         import importlib
 
